pub enum Error {
Params {
expected: usize,
actual: usize,
},
Pool(PoolError),
Query(Error),
Role(Error),
InvalidJson,
Create(CreatePoolError),
Tls(Error),
}Available on crate feature
deadpool only.Expand description
Errors related to pooling or running queries against the Postgres database
Variants
Params
Parameters did not match the number of parameters inferred by statement preparation.
Fields
Pool(PoolError)
Bubbled-up deadpool_postgres connection pool errors.
Query(Error)
Bubbled-up tokio_postgres SQL-level errors within a connection.
Role(Error)
ROLE-setting errors before connections are returned to users.
InvalidJson
JSON-formatted rows could not be properly converted between Postgres’ built-in to_json() output and
serde_json::Value. If this error occurs, it is because of an AS-induced name collision!
Create(CreatePoolError)
Bubbled-up configuration errors from the underlying deadpool_postgres configuration.
Tls(Error)
Available on crate feature
ssl-native-tls only.TLS errors during setup of native_tls SSL connectors.